#include <avr/io.h>
#include <stdio.h>
#include <stdbool.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <math.h>
#include <stdlib.h>
#include "text.h" //where to put all the precoded characters
#include "notes.h"

void send_e();
void draw_symbol(uint8_t x, uint8_t y, uint8_t cs);
void timer_ss();
void clear_display();
void draw_lcd();
void show_goal(int team);
void button_handler();
void timer_stop();
int return_number(int in);

#define A 0
#define B 1
#define C 2
#define D 3
#define E 4
#define F 5
#define G 6
#define H 7
#define I 8
#define J 9
#define K 10
#define L 11
#define M 12
#define N 13
#define O 14
#define P 15
#define Q 16
#define R 17
#define S 18
#define T 19
#define U 20
#define V 21
#define W 22
#define X 23
#define Y 24
#define Z 25
#define EXCL 26
#define FNUTT 27
#define BRAED 28
#define COMMA 29
#define PUNCT 30
#define ZERO 31
#define ONE 32
#define TWO 33
#define THREE 34
#define FOUR 35
#define FIVE 36
#define SIX 37
#define SEVEN 38
#define EIGHT 39
#define NINE 40
#define COLON 41
#define LESS 42
#define EQ 43
#define GREAT 44
#define QUESTION 45
#define HALFCOLON 46
#define HALFCOLON2 49
#define HALFIL 47
#define HALFIR 48
#define CS1 1
#define CS2 2
#define CS3 3
#define DEFAULT_VOLUME 100
#define EMPTY 50
#define SPEAKER 51

/*
 * pa0 - lcd d0 - out
 * pa1 - lcd d1 - out
 * pa2 - lcd d2 - out
 * pa3 - lcd d3 - out
 * pa4 - lcd d4 - out
 * pa5 - lcd d5 - out
 * pa6 - lcd d6 - out
 * pa7 - lcd d7 - out
 *
 * pb0 - buttonc da - in
 * pb1 - buttonc db - in
 * pb2 - buttonc dc - in
 * pb3 - buttonc dd - in
 *
 * pc0 - lcd r/w - out
 * pc1 - lcd d/i(rs) - out
 * pc6 - lcd cs1 - out
 * pc7 - lcd cs2 - out
 *
 * pd0 - lcd rst
 * pd1 - button out enable - out
 * pd2 - button interrupt - in
 * pd3 - sensor interurpt - in
 * pd4 - speaker pwm - out
 * pd5 - g1 sensor - in
 * pd6 - g2 sensor - in
 * pd7 - lcd e - out
 *
 *
 */

// Star Wars
const int starwars[] = 
{
	Ais2,8, Ais2,8, P,16, F3,8, F3,8, P,16, Dis3,16, P,16, D3,16, P,16, C3,16, P,16, Ais3,8,
	Ais3,8, P,16, F3,8, P,16, Dis3,16, P,16, D3,16, P,16, C3,16, P,16, Ais3,8, Ais3,8, P,16,
	F3,8, P,16, Dis3,16, P,16, D3,16, P,16, Dis3,16, P,16, C3,8, C3,8, 
	MUSIC_END
};
// Fur Elise
const int furelise[] = 
{
    e4, 8, d4x, 8, e4, 8, d4x, 8, e4, 8, b3, 8, d4, 8, c4, 8, a3,8, p, 8,
    c3, 8, e3, 8, a3, 8,  b3, 4, p, 8, e3, 8, g3x, 8, b3, 8, c4, 4, p, 8, e3, 8,
    e3, 8, d4x, 8, e4, 8, d4x, 8, e4, 8, b3, 8, d4, 8, c4, 8, a3, 8, p, 8, c3, 8,
    e3, 8, a3, 8, b3, 4, p, 8, e3, 8, c4, 8, b3, 8, a3, 4,
    MUSIC_END
};

// Beatles, Hey Jude
const int Jude[] = {
	G2,8, E2,8, P,16, E2,16, E2,16, G2,16, A2,16, D2,8, P,16, D2,16, E2,16, F2,8, 
	C3,8, C3,16, C3,16, H2,16, G2,16, A2,16, G2,16, F2,16, E2,8, P,16, G2,16, 
	A2,16, A2,8, A2,16, D3,16, C3,16, H2,16, H2,16, C3,16, A2,16, G2,8, P,16, 
	C2,16, D2,16, E2,16, A2,16, A2,16, G2,8,
	MUSIC_END
};

const int goal[] = {
	G2,8, G2,8,G2,8,G2,8,
	MUSIC_END
};

const int love [] = {C2,4, Dis2,4, C2,8, D2,8, C2,8, E2,4, E2,4, F2,4, F2,4, Dis2,8, C2,8, Dis2,8, C2,8, Cis2,8, Cis2,8, C2,8, 
C2,4,Dis2,4, C2,8, Dis2,4, C2,8, E2,8, E2,8, F2,8, F2,8, Dis2,4, C2,8, Dis2,8, C2,8, Cis2,4, 1,4, MUSIC_END};


const int octave[] = {c4, 8, d4, 8, e4, 8, f4, 8, g4, 8, a4, 8, h4, 8, c5, 8, MUSIC_END};

volatile int seconds;
volatile int counter;
volatile int timer_running;
volatile uint8_t g1;
volatile uint8_t g2;
volatile uint8_t period;
volatile uint8_t goal1;
volatile uint8_t goal2;
volatile uint8_t ctw; //the compiler is borked. det hr r en fuling.
volatile uint8_t sjab;
volatile uint8_t volume;
volatile int gdebounce;
volatile uint8_t flag;

void PlayMusic( const int* pMusicNotes /** Pointer to table containing music data */,
				uint8_t tempo /** paying tempo from 0 to 100. Higher value = slower playback*/ )
{
	int duration;
	int note;
	int i;
	uint16_t delay = tempo * 1000*2;

	while( *pMusicNotes )
	{
		note = *pMusicNotes;
		pMusicNotes++;

		duration = *pMusicNotes;
		pMusicNotes++;

		if( p == note )
		{
			//pause, do not generate any sound
			OCR1B = 0;
		}
		else
		{
			//not a pause, generate tone
			OCR1B = volume;

			//set frequency
			ICR1H = (note >> 8);
			ICR1L = note;
		}

		//wait duration
		for(i=0;i<32-duration;i++)
		{
			_delay_loop_2( delay );
		}

	}
	

	//turn off any sound
	OCR1B = 0;
}



void send_e(){ //write to lcd
	PORTD|= (1<<PD7);
	_delay_us (100); //100us delay between e and !e, needed for the lcd to catch up.
	PORTD&= ~(1<<PD7);
	_delay_us (100); //100us delay between !e and e, needed for the lcd to catch up.
}



void timer_ss(){
	timer_running = !timer_running;
	if(timer_running){
		TIMSK |= (1<<TOIE0); //enable timer
	}else{
		TIMSK &= ~(1<<TOIE0); //disable timer
	}
}

void clr_display(){
	uint8_t x;
	uint8_t y;
	PORTC&=~(1<<PC7); //set cs2 = 0
	PORTC&=~(1<<PC6); //set cs1 = 1
	PORTA = 0b01000000; //set y
	send_e();
	for(x = 0; x < 8; x++){
		PORTA = 0b10111000 + x; //set x
		send_e();
		PORTC|=(1<<PC1); //rs = 1
		PORTA = 0b00000000; //data to write
		for( y = 0; y < 64; y++){
			send_e();
		}
		PORTC&=~(1<<PC1); //rs = 0
		send_e();
	}
}

void init_lcd(){
	cli(); //disable interrupts
	PORTD|=(1<<PD0); //sending rst = 1 to lcd. 0 = reset, 1 = normal operation.
	PORTC&=~(1<<PC0); //r/w to lcd
	PORTC&=~(1<<PC1); //rs to lcd
	PORTA = 0b00111111; //display on to lcd
	send_e();//write to lcd
	sei(); //re-enable interrupts
}

void draw_lcd(){ 
	init_lcd();
	clr_display();
	uint8_t i;
	PORTC&=~(1<<PC7); //set cs2 = 1
	PORTC&=~(1<<PC6); //set cs1 = 1
	PORTA = 0b01001100; //set y
	send_e();
	PORTA = 0b10111001; //set x
	send_e();
	PORTA = 0b00000001; //data to write
	PORTC|=(1<<PC1); //rs = 1
	for(i = 0; i < 40; i++){
		send_e();
	}

	PORTC&=~(1<<PC1); //rs = 0

	PORTC&=~(1<<PC1); //rs = 0
	ctw = T; //the compiler is borked. this is one ugly solution.
	draw_symbol(0, 12, CS3);
	draw_symbol(4, 48, CS1);
	ctw = E;
	draw_symbol(0, 20, CS3);
	draw_symbol(4, 8, CS2);	
	ctw = A;
	draw_symbol(0, 28, CS3);
	ctw = M;
	draw_symbol(0, 36, CS3);
	draw_symbol(4, 0, CS2);
	ctw = ONE;
	draw_symbol(0, 44, CS1);
	draw_symbol(2, 0, CS2);
	ctw = TWO;
	draw_symbol(0, 44, CS2);
	ctw = P;
	draw_symbol(2, 56, CS1);
	
	ctw = I;
	draw_symbol(4, 56, CS1);
	ctw = HALFCOLON;
	draw_symbol(5, 60, CS1);
	
	ctw = ZERO;
	draw_symbol(5, 44, CS1);
	draw_symbol(5, 52, CS1);

	draw_symbol(5, 4, CS2);
	draw_symbol(5, 12, CS2);

	draw_symbol(2, 24, CS3);
	draw_symbol(2, 32, CS3);

	ctw = SPEAKER;
	draw_symbol(6, 56, CS2);

}

void draw_symbol(uint8_t x, uint8_t y, uint8_t cs){
	volatile uint8_t i;
	volatile uint8_t j;
	if(cs == 2){
		PORTC&=~(1<<PC7); //set cs2 = 0
		PORTC|=(1<<PC6); //set cs1 = 1
	}else if(cs == 1){
		PORTC&=~(1<<PC6); //set cs1 = 0
		PORTC|=(1<<PC7); //set cs2 = 1
	}else{
		PORTC&=~(1<<PC7); //set cs2 = 1
		PORTC&=~(1<<PC6); //set cs1 = 1
	}
	PORTA = 0b01000000 + y;
	send_e();
	PORTA = 0b10111000 + x;
	send_e();
	PORTC|=(1<<PC1); //rs = 1
	for(i = 0; i < 8; i++){
		sjab = 0b00000000;
		for(j = 0; j < 8; j++){
			if((7 - i - j) >= 1){
				sjab |= ((pgm_read_byte(&fs[ctw][j]) & (0b10000000 >> i)) >> (7-i-j));
			}else{
				sjab |= ((pgm_read_byte(&fs[ctw][j]) & (0b10000000 >> i)) << abs(7-i-j));
			}
		}
		PORTA = sjab;
		send_e();
	}	
	PORTC&=~(1<<PC1); //rs = 0
}


void button_handler(int button_type){ 
	switch(button_type){
		case 0b11111010: timer_ss(); break; //ss button - OK
		case 0b11110011: timer_stop();
						if((g1 >= 0) && (g1 < 99)){ //g1+ button - OK
							g1++;
						}else if(g1 >= 99){
							g1 = 0;
						} break; 
		case 0b11110111: timer_stop(); 
						if((g1 > 0) && (g1 < 100)){ //g1- button - OK
							g1--;
						}else if(g1 <= 0){
							g1 = 99;
						} break;
		case 0b11110010: timer_stop();
						if((g2 >= 0) && (g2 < 99)){ //g2+ button - OK
							g2++;
						}else if(g2 >= 99){
							g2 = 0;
						} break;
		case 0b11110110: timer_stop();
						if((g2 > 0) && (g2 < 100)){ //g2- button - OK
							g2--;
						}else if(g2 <= 0){
							g2 = 99;
						}
						break;
		case 0b11110001: timer_stop();
						if(seconds < 5940){ //minplus button - OK
							seconds += 60;
						} 
						break; 
		case 0b11110101: timer_stop(); 
						if(seconds > 60){ //minmin button - OK
				     		seconds -= 60;
			     		}else{
				    		seconds = 0;
			     		} 
			     		break;
		case 0b11110100: timer_stop(); 
					if(period < 3){ //period button - OK
				     period++;
			     }else{
				     period = 1;
			     }
			     break;
		case 0b11111000: TCNT0 = 0; //RESET OK! //ccw
				 g1 = g2 = counter = seconds = gdebounce = flag = 0;
				 timer_running = false; 
				 period = 1; 
				 draw_lcd();
				 timer_stop();
				 break; //reset
		case 0b11111011: if(volume == 100){		//volume button. sets speaker on/off
							volume = 0;
							ctw = X;
							draw_symbol(6, 56, CS2);
						}else{
								volume = 100;
								ctw = SPEAKER;
								draw_symbol(6, 56, CS2);
								}
								break;
		case 0b11111110: timer_stop();
						if(gdebounce != seconds){ 
							g1++;
							goal1 = 1;
							gdebounce = seconds;
						}
						break;
		case 0b11111111: timer_stop();
						if(gdebounce != seconds){
							g2++;
							goal2 = 1;
							gdebounce = seconds;
						}
						break;
		default: break;
	}
}


int return_number(int in){
	volatile uint8_t rv;
	switch(in){
		case 0: rv = ZERO; break;
		case 1: rv = ONE; break;
		case 2: rv = TWO; break;
		case 3: rv = THREE; break;
		case 4: rv = FOUR; break;
		case 5: rv = FIVE; break;
		case 6: rv = SIX; break;
		case 7: rv = SEVEN; break;
		case 8: rv = EIGHT; break;
		case 9: rv = NINE; break;
		default: break;
	}
	return rv;
}


void update_lcd(){ 
	//goals
	if(g1 >= 10){ 
		ctw = return_number(g1/10);
		draw_symbol(2, 24, CS1);
		ctw = return_number(g1%10);
		draw_symbol(2, 32, CS1);
	} else{
		ctw = ZERO;
		draw_symbol(2, 24, CS1);
		ctw = return_number(g1);
		draw_symbol(2, 32, CS1);
	}
	if(g2 >= 10){
		ctw = return_number(g2/10);
		draw_symbol(2, 24, CS2);
		ctw = return_number(g2%10);
		draw_symbol(2, 32, CS2);
	} else{
		ctw = ZERO;
		draw_symbol(2, 24, CS2);
		ctw = return_number(g2);
		draw_symbol(2, 32, CS2);
	}
	//time
	volatile uint8_t min;
	min = seconds/60;
	if(seconds > 59){
		if(seconds > 599){
			ctw = return_number(min/10);
			draw_symbol(5, 44, CS1);
			ctw = return_number(min%10);
			draw_symbol(5, 52, CS1);
			ctw = return_number((seconds-(min*60))/10);
			draw_symbol(5, 4, CS2);
			ctw = return_number((seconds-(min*60))%10);
			draw_symbol(5, 12, CS2);
		}else{
			ctw = ZERO;
			draw_symbol(5, 44, CS1);
			ctw = return_number(min);
			draw_symbol(5, 52,  CS1);
			ctw = return_number((seconds-(min*60))/10);
			draw_symbol(5, 4, CS2);
			ctw = return_number((seconds-(min*60))%10);
			draw_symbol(5, 12, CS2);
		}
	} else {
			ctw = ZERO;
			draw_symbol(5, 44, CS1);
			ctw = ZERO;
			draw_symbol(5, 52, CS1);
			ctw = return_number(seconds/10);
			draw_symbol(5, 4, CS2);
			ctw = return_number(seconds%10);
			draw_symbol(5, 12, CS2);
	}
	//period
	ctw = return_number(period);
	draw_symbol(2, 0, CS2);
}

void show_end(){
	flag = 1;
	if(period == 3){
		ctw = G;
		draw_symbol(7, 30, CS1);
		ctw = A;
		draw_symbol(7, 38, CS1);
		ctw = M;
		draw_symbol(7, 46, CS1);
		ctw = E;
		draw_symbol(7, 54, CS1);
		ctw = O;
		draw_symbol(7, 2, CS2);
		ctw = V;
		draw_symbol(7, 10, CS2);
		ctw = E;
		draw_symbol(7, 18, CS2);
		ctw = R;
		draw_symbol(7, 26, CS2);
	}else{
		ctw = T;
		draw_symbol(7, 38, CS1);
		ctw = I;
		draw_symbol(7, 46, CS1);
		ctw = M;
		draw_symbol(7, 54, CS1);
		ctw = E;
		draw_symbol(7, 0, CS2);
		ctw = U;
		draw_symbol(7, 12, CS2);
		ctw = P;
		draw_symbol(7, 20, CS2);
	}
}

void show_goal(int team){
	uint8_t i;
	for(i = 1; i < 14; i++){
		_delay_ms (2000); //speed of the scrolling text
		if( i < 9){
			ctw = L;
			draw_symbol(7, (i*8)-8, team);
		}
		if(i > 1 && i < 10){
			ctw = A;
			draw_symbol(7, (i*8)-16,team);
		}
		if(i > 2 && i < 11){
			ctw = O;
			draw_symbol(7, (i*8)-24,team);
		}
		if(i > 3 && i < 12){
			ctw = G;
			draw_symbol(7, (i*8)-32,team);
		}
		if(i > 4 && i < 13){
			ctw = EMPTY;
			draw_symbol(7, (i*8)-40,team);
		}
	}
}

void play_goal_sound(){

PlayMusic( goal,8);

}

void play_end_sound(){
PlayMusic( starwars, 17 );
//PlayMusic( furelise, 20 );
//PlayMusic( Jude, 35 );


}

void timer_stop(){
	TIMSK &= ~(1<<TOIE0); //disable timer
	timer_running = 0;
}

ISR(INT0_vect){ //button interrupt
	PORTD|=(1<<PD1);
	button_handler(PINB); 
	PORTD&=~(1<<PD1);
}

ISR(INT1_vect){ //goal interrupt
	TIMSK &= ~(1<<TOIE0); //disable timer
	timer_running = 0;
	if((PIND & (1 << PD5)) == 0){
		g1++;
			goal1 = 1;
	}else if(PIND & (1 << PD6)){
		g2++;	
			goal2 = 1;
	}
}

ISR(TIMER0_OVF_vect){ //timer interrupt
	counter++;
}


void pwm_init(){
	// Configure OC1B pin as output

	DDRD |= _BV(DDD4); //OC1B as output

	// timer1 configuration (for PWM)
    TCCR1A |= _BV(COM1B1);  // Clear OC1A/OC1B on compare match

	
	TCCR1B |= _BV(WGM13) 	//mode 8, PWM, Phase and Frequency Correct (TOP value is ICR1)
		   |_BV(CS11); 		//prescaler(8)
}



int main(void){
	
	
	

	DDRA = 0xFF; //set all pins for output
	DDRB = 0x00; //set all pins as input
	DDRC = 0b11000011; //set 0,1,6,7 as output, rest as input
	DDRD = 0b10010011; //set different in/out

	PORTB = 0xFF; //enable pullup

	PORTD|=(1<<PD2);
	PORTD&= ~(1<<PD3); //&= ~
	PORTD&= ~(1<<PD5);
	PORTD&= ~(1<<PD6);

	GICR |= 1<<INT0;
	GICR |= 1<<INT1;

	MCUCR	|= 0x0F;	// set trigger for INT0 and INT1

	volume = 100;
	pwm_init();
	//PlayMusic( love, 62 );
	//PlayMusic( furelise, 20 );
	seconds = 0;
	g1 = 0;
	g2 = 0;
	counter = 0;
	period = 1;
	timer_running = 0;
	goal1 = 0;
	goal2 = 0;
	gdebounce = 0;
	flag = 0;
	draw_lcd();

//prescaler, timer0. FCPU/64, it will have a freq of 250kHz which means 1ms per tick
	TCCR0 |= (1<<CS02) | (1<<CS00);
	
	TCNT0 = 0; //start value of counter	

	sei();
	while(1){
		if(goal1){
			
			play_goal_sound();

			show_goal(1);

			goal1 = 0;
		}
		if(goal2){
			
			play_goal_sound();

			show_goal(2);

			goal2 = 0;
		}
		if(timer_running && flag){
				ctw = EMPTY;
				uint8_t i;
				for(i = 1; i < 9; i++){
					draw_symbol(7, i*8, CS3);
				}
				flag = 0;
	}

		if(counter >= 61){
			counter = 0;
			if(seconds <= 0){
				TIMSK &= ~(1<<TOIE0); //disable timer
				timer_running = 0;
				gdebounce = 0;
				show_end();
				play_end_sound();
			} else{
				seconds--;
			}
		}
		cli();
		update_lcd();
		sei();
	}
	
}


